//////////////////////////////////////////////////////
// Direct3D.cpp
//////////////////////////////////////////////////////

#include "direct3d.h"

//////////////////////////////////////////////////////
// CDirect3D()
//////////////////////////////////////////////////////
CDirect3D::CDirect3D(void)
{
    m_pD3D = NULL;
    m_pD3DDevice = NULL;
    m_pBackBuffer = NULL;
    strcpy(m_szErrorMsg, "No Error");
    m_pTownExtTileSurface = NULL;
    m_pTownIntTileSurface = NULL;
    m_pDungeonTileSurface = NULL;
    m_pWildernessTileSurface = NULL;
    m_pCurrentTileSurface = NULL;
    m_pPlayerTile = NULL;
    m_pTexture = NULL;
    for (int x=0; x<4; ++x)
        m_pMonsterTexture[x] = NULL;
}

//////////////////////////////////////////////////////
// ~CDirect3D()
//////////////////////////////////////////////////////
CDirect3D::~CDirect3D(void)
{
    CleanUp();
}

//////////////////////////////////////////////////////
// SetWindowHandle()
//////////////////////////////////////////////////////
void CDirect3D::SetWindowHandle(HWND hWnd)
{
    m_hWnd = hWnd;
}

//////////////////////////////////////////////////////
// InitD3D()
//////////////////////////////////////////////////////
HRESULT CDirect3D::InitD3D()
{
    HRESULT hResult = CreateD3DObject();
    if (hResult != D3D_OK)
        return hResult;
    hResult = CheckDisplayMode();
    if (hResult != D3D_OK)
        return hResult;
    hResult = CreateD3DDevice();
    if (hResult != D3D_OK)
        return hResult;
    hResult = CreateSurfaces();
    if (hResult != D3D_OK)
        return hResult;
    hResult = m_pD3DDevice->GetBackBuffer(0,
        D3DBACKBUFFER_TYPE_MONO, &m_pBackBuffer);
    m_pCurrentTileSurface = m_pTownExtTileSurface;
    m_pD3DDevice->Clear(0, 0, D3DCLEAR_TARGET, 
        D3DCOLOR_XRGB(0,0,255), 0, 0);
    m_pD3DDevice->Present(NULL, NULL, NULL, NULL);
    return hResult;
}

//////////////////////////////////////////////////////
// CreateD3DObject()
//////////////////////////////////////////////////////
HRESULT CDirect3D::CreateD3DObject()
{
    m_pD3D = Direct3DCreate8(D3D_SDK_VERSION);
    if (m_pD3D == NULL)
    {
        MessageBox(m_hWnd, "Couldn't create DirectX object.",
            "DirectX Error", MB_OK);
        strcpy(m_szErrorMsg, "CreateD3DObject()");
        return E_FAIL;
    }
    return D3D_OK;
}

//////////////////////////////////////////////////////
// CheckDisplayMode()
//////////////////////////////////////////////////////
HRESULT CDirect3D::CheckDisplayMode()
{
    HRESULT hResult = m_pD3D->CheckDeviceType(D3DADAPTER_DEFAULT, 
        D3DDEVTYPE_HAL, D3DFMT_X8R8G8B8, D3DFMT_X8R8G8B8, FALSE);
    if (hResult != D3D_OK)
    {
        MessageBox(m_hWnd, "The required display mode is \
not available on this system.",
            "DirectX Error", MB_OK);
        strcpy(m_szErrorMsg, "CheckDisplayMode()");
        return hResult;
    }
    return D3D_OK;
}

//////////////////////////////////////////////////////
// CreateD3DDevice()
//////////////////////////////////////////////////////
HRESULT CDirect3D::CreateD3DDevice()
{
    D3DPRESENT_PARAMETERS D3DPresentParams;
    ZeroMemory(&D3DPresentParams, sizeof(D3DPRESENT_PARAMETERS));
    D3DPresentParams.Windowed = FALSE;
    D3DPresentParams.BackBufferCount = 1;
    D3DPresentParams.BackBufferWidth = 640;
    D3DPresentParams.BackBufferHeight = 480;
    D3DPresentParams.BackBufferFormat = D3DFMT_X8R8G8B8;
    D3DPresentParams.SwapEffect = D3DSWAPEFFECT_DISCARD;
    D3DPresentParams.hDeviceWindow = m_hWnd;
    HRESULT hResult = m_pD3D->CreateDevice(D3DADAPTER_DEFAULT, 
        D3DDEVTYPE_HAL, m_hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING,
        &D3DPresentParams, &m_pD3DDevice);
    if (FAILED(hResult))
    {
        MessageBox(m_hWnd, "Failed to create Direct3D device.",
            "DirectX Error", MB_OK);
        strcpy(m_szErrorMsg, "CreateD3DDevice()");
        return hResult;
    }
    return hResult;
}

//////////////////////////////////////////////////////
// CreateSurfaces()
//////////////////////////////////////////////////////
HRESULT CDirect3D::CreateSurfaces()
{
    HRESULT hResult = m_pD3DDevice->CreateImageSurface(256, 256, 
        D3DFMT_X8R8G8B8, &m_pTownExtTileSurface);
    if (FAILED(hResult))
    {
        strcpy(m_szErrorMsg, "Town ext tile surface not created.");
        return hResult;
    }

    hResult = D3DXLoadSurfaceFromFile(m_pTownExtTileSurface, NULL, NULL,
        "Graphics\\Town01Ext.bmp", NULL, D3DX_DEFAULT, 0, NULL);
    if (FAILED(hResult))
    {
        strcpy(m_szErrorMsg, "Error loading ext town surface.");
        return hResult;
    }

    hResult = m_pD3DDevice->CreateImageSurface(256, 256, 
        D3DFMT_X8R8G8B8, &m_pTownIntTileSurface);
    if (FAILED(hResult))
    {
        strcpy(m_szErrorMsg, "Town int tile surface not created.");
        return hResult;
    }

    hResult = D3DXLoadSurfaceFromFile(m_pTownIntTileSurface, NULL, NULL,
        "Graphics\\Town01Int.bmp", NULL, D3DX_DEFAULT, 0, NULL);
    if (FAILED(hResult))
    {
        strcpy(m_szErrorMsg, "Error loading int town surface.");
        return hResult;
    }

    hResult = m_pD3DDevice->CreateImageSurface(256, 256, 
        D3DFMT_X8R8G8B8, &m_pDungeonTileSurface);
    if (FAILED(hResult))
    {
        strcpy(m_szErrorMsg, "Dungeon tile surface not created.");
        return hResult;
    }

    hResult = D3DXLoadSurfaceFromFile(m_pDungeonTileSurface, NULL, NULL,
        "Graphics\\Dungeon01.bmp", NULL, D3DX_DEFAULT, 0, NULL);
    if (FAILED(hResult))
    {
        strcpy(m_szErrorMsg, "Error loading dungeon surface.");
        return hResult;
    }

    hResult = m_pD3DDevice->CreateImageSurface(256, 256, 
        D3DFMT_X8R8G8B8, &m_pWildernessTileSurface);
    if (FAILED(hResult))
    {
        strcpy(m_szErrorMsg, "Wilderness tile surface not created.");
        return hResult;
    }

    hResult = D3DXLoadSurfaceFromFile(m_pWildernessTileSurface, NULL, NULL,
        "Graphics\\Wilderness.bmp", NULL, D3DX_DEFAULT, 0, NULL);
    if (FAILED(hResult))
    {
        strcpy(m_szErrorMsg, "Error loading wilderness surface.");
        return hResult;
    }

    hResult = m_pD3DDevice->CreateImageSurface(32, 32, 
        D3DFMT_X8R8G8B8, &m_pPlayerTile);
    if (FAILED(hResult))
    {
        strcpy(m_szErrorMsg, "Player tile surface not created.");
        return hResult;
    }

    hResult = D3DXLoadSurfaceFromFile(m_pPlayerTile, NULL, NULL,
        "Graphics\\Player.bmp", NULL, D3DX_DEFAULT, 0, NULL);
    if (FAILED(hResult))
    {
        strcpy(m_szErrorMsg, "Error loading player tile surface.");
        return hResult;
    }

    hResult = D3DXCreateTextureFromFileEx(m_pD3DDevice, "Graphics\\Plaque.bmp",
        D3DX_DEFAULT, D3DX_DEFAULT, 1, 0, D3DFMT_A8R8G8B8,
        D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, 0xFF000000,
        NULL, NULL, &m_pTexture);
    if (FAILED(hResult))
    {
        strcpy(m_szErrorMsg, "Error loading texture surface.");
        return hResult;
    }
    return hResult;
}

//////////////////////////////////////////////////////
// CleanUp()
//////////////////////////////////////////////////////
void CDirect3D::CleanUp()
{
    for (int x=0; x<4; ++x)
    {
        if (m_pMonsterTexture[x])
        {
            m_pMonsterTexture[x]->Release();
            m_pMonsterTexture[x] = NULL;
        }
    }

    if (m_pTexture)
        m_pTexture->Release();

    if (m_pTownExtTileSurface) 
        m_pTownExtTileSurface->Release();

    if (m_pTownIntTileSurface) 
        m_pTownIntTileSurface->Release();

    if (m_pDungeonTileSurface) 
        m_pDungeonTileSurface->Release();

    if (m_pWildernessTileSurface) 
        m_pWildernessTileSurface->Release();

    if (m_pPlayerTile) 
        m_pPlayerTile->Release();

    if (m_pBackBuffer)
        m_pBackBuffer->Release();

    if (m_pD3DDevice) 
        m_pD3DDevice->Release();

    if (m_pD3D)
        m_pD3D->Release();
}

//////////////////////////////////////////////////////
// GetBackBuffer()
//////////////////////////////////////////////////////
IDirect3DSurface8* CDirect3D::GetBackBuffer()
{
    return m_pBackBuffer;
}

//////////////////////////////////////////////////////
// Getdevice()
//////////////////////////////////////////////////////
LPDIRECT3DDEVICE8 CDirect3D::GetDevice()
{
    return m_pD3DDevice;
}

//////////////////////////////////////////////////////
// GetErrorString()
//////////////////////////////////////////////////////
char* CDirect3D::GetErrorString(void)
{
    return m_szErrorMsg;
}

//////////////////////////////////////////////////////
// GetCurrentTileSurface()
//////////////////////////////////////////////////////
IDirect3DSurface8* CDirect3D::GetCurrentTileSurface()
{
    return m_pCurrentTileSurface;
}

//////////////////////////////////////////////////////
// GetPlayerTile()
//////////////////////////////////////////////////////
IDirect3DSurface8* CDirect3D::GetPlayerTile()
{
    return m_pPlayerTile;
}

//////////////////////////////////////////////////////
// SetCurrentTileSurface()
//////////////////////////////////////////////////////
void CDirect3D::SetCurrentTileSurface(IDirect3DSurface8* pSurface)
{
    m_pCurrentTileSurface = pSurface;
}

//////////////////////////////////////////////////////
// GetTownExtTileSurface()
//////////////////////////////////////////////////////
IDirect3DSurface8* CDirect3D::GetTownExtTileSurface()
{
    return m_pTownExtTileSurface;
}

//////////////////////////////////////////////////////
// GetTownIntTileSurface()
//////////////////////////////////////////////////////
IDirect3DSurface8* CDirect3D::GetTownIntTileSurface()
{
    return m_pTownIntTileSurface;
}

//////////////////////////////////////////////////////
// GetWildernessIntTileSurface()
//////////////////////////////////////////////////////
IDirect3DSurface8* CDirect3D::GetWildernessTileSurface()
{
    return m_pWildernessTileSurface;
}

//////////////////////////////////////////////////////
// GetTexture()
//////////////////////////////////////////////////////
IDirect3DTexture8* CDirect3D::GetTexture()
{
    return m_pTexture;
}

//////////////////////////////////////////////////////
// GetDungeonTileSurface()
//////////////////////////////////////////////////////
IDirect3DSurface8* CDirect3D::GetDungeonTileSurface()
{
    return m_pDungeonTileSurface;
}

//////////////////////////////////////////////////////
// GetMonsterTexture()
//////////////////////////////////////////////////////
IDirect3DTexture8* CDirect3D::GetMonsterTexture(int iFrame)
{
    return m_pMonsterTexture[iFrame];
}

//////////////////////////////////////////////////////
// CreateMonsterTextures()
//////////////////////////////////////////////////////
void CDirect3D::CreateMonsterTextures(char* idle1, 
    char* idle2, char* attack, char* hit)
{
    D3DXCreateTextureFromFileEx(m_pD3DDevice, idle1,
        D3DX_DEFAULT, D3DX_DEFAULT, 1, 0, D3DFMT_A8R8G8B8,
        D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, 0xFF000000,
        NULL, NULL, &m_pMonsterTexture[0]);
    D3DXCreateTextureFromFileEx(m_pD3DDevice, idle2,
        D3DX_DEFAULT, D3DX_DEFAULT, 1, 0, D3DFMT_A8R8G8B8,
        D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, 0xFF000000,
        NULL, NULL, &m_pMonsterTexture[1]);
    D3DXCreateTextureFromFileEx(m_pD3DDevice, attack,
        D3DX_DEFAULT, D3DX_DEFAULT, 1, 0, D3DFMT_A8R8G8B8,
        D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, 0xFF000000,
        NULL, NULL, &m_pMonsterTexture[2]);
    D3DXCreateTextureFromFileEx(m_pD3DDevice, hit,
        D3DX_DEFAULT, D3DX_DEFAULT, 1, 0, D3DFMT_A8R8G8B8,
        D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, 0xFF000000,
        NULL, NULL, &m_pMonsterTexture[3]);
}

//////////////////////////////////////////////////////
// ReleaseMonsterTextures()
//////////////////////////////////////////////////////
void CDirect3D::ReleaseMonsterTextures()
{
    for (int x=0; x<4; ++x)
    {
        if (m_pMonsterTexture[x])
        {
            m_pMonsterTexture[x]->Release();
            m_pMonsterTexture[x] = NULL;
        }
    }
}
