//////////////////////////////////////////////////////
// DirectSound.cpp
//////////////////////////////////////////////////////

#include "directsound.h"

//////////////////////////////////////////////////////
// CDirectSound()
//////////////////////////////////////////////////////
CDirectSound::CDirectSound(void)
{
    m_pDirectSoundObj = NULL;
    m_pMonsterDieSound = NULL;
    m_pMonsterHitSound = NULL;
    m_pPlayerHitSound = NULL;
    m_pFootstepSound = NULL;
    m_pItemOpenSound = NULL;
}

//////////////////////////////////////////////////////
// ~CDirectSound()
//////////////////////////////////////////////////////
CDirectSound::~CDirectSound(void)
{
    CleanUp();
}

//////////////////////////////////////////////////////
// SetWindowHandle()
//////////////////////////////////////////////////////
void CDirectSound::SetWindowHandle(HWND hWnd)
{
    m_hWnd = hWnd;
}

//////////////////////////////////////////////////////
// CleanUp()
//////////////////////////////////////////////////////
void CDirectSound::CleanUp()
{
    if (m_pMonsterDieSound)
        m_pMonsterDieSound->Release();
    if (m_pMonsterHitSound)
        m_pMonsterHitSound->Release();
    if (m_pPlayerHitSound)
        m_pPlayerHitSound->Release();
    if (m_pFootstepSound)
        m_pFootstepSound->Release();
    if (m_pItemOpenSound)
        m_pItemOpenSound->Release();
    if (m_pDirectSoundObj)
        m_pDirectSoundObj->Release();
}

//////////////////////////////////////////////////////
// InitDirectSound()
//////////////////////////////////////////////////////
HRESULT CDirectSound::InitDirectSound()
{
    HRESULT hResult = DirectSoundCreate8(NULL, &m_pDirectSoundObj, NULL);
    if (hResult != DS_OK)
        return hResult;

    hResult = m_pDirectSoundObj->SetCooperativeLevel(m_hWnd, DSSCL_NORMAL);
    if (hResult != DS_OK)
        return hResult;

    hResult = CreateSoundBuffers();
    if (hResult != DS_OK)
        return hResult;

    return hResult;
}

//////////////////////////////////////////////////////
// CreateSoundBuffers()
//////////////////////////////////////////////////////
HRESULT CDirectSound::CreateSoundBuffers()
{
    HRESULT hResult = LoadSoundEffect("Sounds\\MonsterDie.wav", &m_pMonsterDieSound);
    if (hResult != DS_OK)
        return hResult;
    hResult = LoadSoundEffect("Sounds\\MonsterHit.wav", &m_pMonsterHitSound);
    if (hResult != DS_OK)
        return hResult;
    hResult = LoadSoundEffect("Sounds\\PlayerHit.wav", &m_pPlayerHitSound);
    if (hResult != DS_OK)
        return hResult;
    hResult = LoadSoundEffect("Sounds\\Footstep.wav", &m_pFootstepSound);
    if (hResult != DS_OK)
        return hResult;
    hResult = LoadSoundEffect("Sounds\\ItemOpen.wav", &m_pItemOpenSound);
    return hResult;
}

//////////////////////////////////////////////////////
// PlaySound()
//////////////////////////////////////////////////////
HRESULT CDirectSound::PlaySound(int iSoundID)
{
    IDirectSoundBuffer* pBuf;
    switch (iSoundID)
    {
    case PLAYERHIT:
        pBuf = m_pPlayerHitSound;
        break;
    case MONSTERHIT:
        pBuf = m_pMonsterHitSound;
        break;
    case MONSTERDIES:
        pBuf = m_pMonsterDieSound;
        break;
    case FOOTSTEP:
        pBuf = m_pFootstepSound;
        break;
    case ITEMOPEN:
        pBuf = m_pItemOpenSound;
        break;
    }
    HRESULT hResult = pBuf->SetCurrentPosition(0);
    if (hResult != DS_OK)
        return hResult;

	hResult = pBuf->Play(0, 0, 0);
    if (hResult != DS_OK)
        return hResult;

    return hResult;
}

//////////////////////////////////////////////////////
// LoadSoundEffect()
//////////////////////////////////////////////////////
HRESULT CDirectSound::LoadSoundEffect(char* pstrFileName, 
    IDirectSoundBuffer** ppSoundBuf)
{
    CWave* pWave = new CWave(pstrFileName);
	if (!pWave->WaveOK())
	{
		MessageBox(m_hWnd, "Could not load wave file.", "Error", MB_OK);
        PostQuitMessage(WM_QUIT);
		return 0;
	}

    LPWAVEFORMATEX pWaveFormatEx = pWave->GetWaveFormatPtr();
    char* pWaveData = pWave->GetWaveDataPtr();
    DWORD dwWaveSize = pWave->GetWaveSize();

    DSBUFFERDESC dsBufferDesc;
    memset(&dsBufferDesc, 0, sizeof(DSBUFFERDESC));
    dsBufferDesc.dwSize = sizeof(DSBUFFERDESC);
    dsBufferDesc.dwBufferBytes = dwWaveSize;
    dsBufferDesc.lpwfxFormat = (LPWAVEFORMATEX) pWaveFormatEx;

    IDirectSoundBuffer* pDSBuf;
	HRESULT hResult = m_pDirectSoundObj->
		CreateSoundBuffer(&dsBufferDesc, &pDSBuf, NULL);
    if (hResult != DS_OK)
        return hResult;

    LPVOID pSoundBlock1;
    LPVOID pSoundBlock2;
    DWORD dwBytesSoundBlock1;
    DWORD dwBytesSoundBlock2;
    hResult = pDSBuf->Lock(0, dwWaveSize,
        &pSoundBlock1, &dwBytesSoundBlock1,
        &pSoundBlock2, &dwBytesSoundBlock2, 0);
    if (hResult != DS_OK)
        return hResult;

    memcpy((void*)pSoundBlock1, pWaveData, dwWaveSize);

	pDSBuf->Unlock(pSoundBlock1, dwBytesSoundBlock1,
            pSoundBlock2, dwBytesSoundBlock2);

	delete pWave;
    *ppSoundBuf = pDSBuf;
    return hResult;
}
